/**
 * DevExtreme (client_exporter/xlsx/xlsx_cell_format_helper.js)
 * Version: 18.2.0 (build 18278)
 * Build date: Fri Oct 05 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
var _type = require("../../core/utils/type");
var _type2 = _interopRequireDefault(_type);
var _xlsx_tag_helper = require("./xlsx_tag_helper");
var _xlsx_tag_helper2 = _interopRequireDefault(_xlsx_tag_helper);
var _xlsx_cell_alignment_helper = require("./xlsx_cell_alignment_helper");
var _xlsx_cell_alignment_helper2 = _interopRequireDefault(_xlsx_cell_alignment_helper);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var xlsxCellFormatHelper = {
    tryCreateTag: function(sourceObj, sharedItemsContainer) {
        var result = null;
        if (_type2.default.isDefined(sourceObj)) {
            var numberFormatId = void 0;
            if (_type2.default.isDefined(sourceObj.numberFormat) && _type2.default.isNumeric(sourceObj.numberFormat)) {
                numberFormatId = sourceObj.numberFormat
            } else {
                numberFormatId = sharedItemsContainer.registerNumberFormat(sourceObj.numberFormat)
            }
            result = {
                numberFormatId: numberFormatId,
                alignment: _xlsx_cell_alignment_helper2.default.tryCreateTag(sourceObj.alignment),
                fontId: sharedItemsContainer.registerFont(sourceObj.font),
                fillId: sharedItemsContainer.registerFill(sourceObj.fill)
            };
            if (xlsxCellFormatHelper.isEmpty(result)) {
                result = null
            }
        }
        return result
    },
    areEqual: function(leftTag, rightTag) {
        return xlsxCellFormatHelper.isEmpty(leftTag) && xlsxCellFormatHelper.isEmpty(rightTag) || _type2.default.isDefined(leftTag) && _type2.default.isDefined(rightTag) && leftTag.fontId === rightTag.fontId && leftTag.numberFormatId === rightTag.numberFormatId && leftTag.fillId === rightTag.fillId && _xlsx_cell_alignment_helper2.default.areEqual(leftTag.alignment, rightTag.alignment)
    },
    isEmpty: function(tag) {
        return !_type2.default.isDefined(tag) || !_type2.default.isDefined(tag.fontId) && !_type2.default.isDefined(tag.numberFormatId) && !_type2.default.isDefined(tag.fillId) && _xlsx_cell_alignment_helper2.default.isEmpty(tag.alignment)
    },
    toXml: function(tag) {
        var isAlignmentEmpty = _xlsx_cell_alignment_helper2.default.isEmpty(tag.alignment);
        var applyNumberFormat = void 0;
        if (_type2.default.isDefined(tag.numberFormatId)) {
            applyNumberFormat = tag.numberFormatId > 0 ? 1 : 0
        }
        return _xlsx_tag_helper2.default.toXml("xf", {
            xfId: 0,
            applyAlignment: isAlignmentEmpty ? null : 1,
            fontId: tag.fontId,
            applyNumberFormat: applyNumberFormat,
            fillId: tag.fillId,
            numFmtId: tag.numberFormatId
        }, isAlignmentEmpty ? null : _xlsx_cell_alignment_helper2.default.toXml(tag.alignment))
    }
};
exports.default = xlsxCellFormatHelper;
