/**
 * DevExtreme (client_exporter/xlsx/xlsx_file.js)
 * Version: 18.2.0 (build 18278)
 * Build date: Fri Oct 05 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _type = require("../../core/utils/type");
var _type2 = _interopRequireDefault(_type);
var _xlsx_tag_helper = require("./xlsx_tag_helper");
var _xlsx_tag_helper2 = _interopRequireDefault(_xlsx_tag_helper);
var _xlsx_cell_format_helper = require("./xlsx_cell_format_helper");
var _xlsx_cell_format_helper2 = _interopRequireDefault(_xlsx_cell_format_helper);
var _xlsx_fill_helper = require("./xlsx_fill_helper");
var _xlsx_fill_helper2 = _interopRequireDefault(_xlsx_fill_helper);
var _xlsx_font_helper = require("./xlsx_font_helper");
var _xlsx_font_helper2 = _interopRequireDefault(_xlsx_font_helper);
var _xlsx_number_format_helper = require("./xlsx_number_format_helper");
var _xlsx_number_format_helper2 = _interopRequireDefault(_xlsx_number_format_helper);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}
var XlsxFile = function() {
    function XlsxFile() {
        _classCallCheck(this, XlsxFile);
        this._cellFormatTags = [];
        this._fillTags = [];
        this._fontTags = [];
        this._numberFormatTags = [];
        this._fillTags.push(_xlsx_fill_helper2.default.tryCreateTag({
            patternFill: {
                patternType: "none"
            }
        }))
    }
    _createClass(XlsxFile, [{
        key: "registerCellFormat",
        value: function(cellFormat) {
            var result = void 0;
            var cellFormatTag = _xlsx_cell_format_helper2.default.tryCreateTag(cellFormat, {
                registerFill: this.registerFill.bind(this),
                registerFont: this.registerFont.bind(this),
                registerNumberFormat: this.registerNumberFormat.bind(this)
            });
            if (_type2.default.isDefined(cellFormatTag)) {
                for (var i = 0; i < this._cellFormatTags.length; i++) {
                    if (_xlsx_cell_format_helper2.default.areEqual(this._cellFormatTags[i], cellFormatTag)) {
                        result = i;
                        break
                    }
                }
                if (void 0 === result) {
                    result = this._cellFormatTags.push(cellFormatTag) - 1
                }
            }
            return result
        }
    }, {
        key: "generateCellFormatsXml",
        value: function() {
            var cellFormatTagsAsXmlStringsArray = this._cellFormatTags.map(function(tag) {
                return _xlsx_cell_format_helper2.default.toXml(tag)
            });
            return _xlsx_tag_helper2.default.toXml("cellXfs", {
                count: cellFormatTagsAsXmlStringsArray.length
            }, cellFormatTagsAsXmlStringsArray.join(""))
        }
    }, {
        key: "registerFill",
        value: function(fill) {
            var result = void 0;
            var fillTag = _xlsx_fill_helper2.default.tryCreateTag(fill);
            if (_type2.default.isDefined(fillTag)) {
                for (var i = 0; i < this._fillTags.length; i++) {
                    if (_xlsx_fill_helper2.default.areEqual(this._fillTags[i], fillTag)) {
                        result = i;
                        break
                    }
                }
                if (void 0 === result) {
                    if (this._fillTags.length < 2) {
                        this._fillTags.push(_xlsx_fill_helper2.default.tryCreateTag({
                            patternFill: {
                                patternType: "Gray125"
                            }
                        }))
                    }
                    result = this._fillTags.push(fillTag) - 1
                }
            }
            return result
        }
    }, {
        key: "generateFillsXml",
        value: function() {
            var tagsAsXmlStringsArray = this._fillTags.map(function(tag) {
                return _xlsx_fill_helper2.default.toXml(tag)
            });
            return _xlsx_tag_helper2.default.toXml("fills", {
                count: tagsAsXmlStringsArray.length
            }, tagsAsXmlStringsArray.join(""))
        }
    }, {
        key: "registerFont",
        value: function(font) {
            var result = void 0;
            var fontTag = _xlsx_font_helper2.default.tryCreateTag(font);
            if (_type2.default.isDefined(fontTag)) {
                for (var i = 0; i < this._fontTags.length; i++) {
                    if (_xlsx_font_helper2.default.areEqual(this._fontTags[i], fontTag)) {
                        result = i;
                        break
                    }
                }
                if (void 0 === result) {
                    result = this._fontTags.push(fontTag) - 1
                }
            }
            return result
        }
    }, {
        key: "generateFontsXml",
        value: function() {
            var xmlStringsArray = this._fontTags.map(function(tag) {
                return _xlsx_font_helper2.default.toXml(tag)
            });
            return _xlsx_tag_helper2.default.toXml("fonts", {
                count: xmlStringsArray.length
            }, xmlStringsArray.join(""))
        }
    }, {
        key: "_convertNumberFormatIndexToId",
        value: function(index) {
            var CUSTOM_FORMAT_ID_START_VALUE = 165;
            return CUSTOM_FORMAT_ID_START_VALUE + index
        }
    }, {
        key: "registerNumberFormat",
        value: function(numberFormat) {
            var result = void 0;
            var tag = _xlsx_number_format_helper2.default.tryCreateTag(numberFormat);
            if (_type2.default.isDefined(tag)) {
                for (var i = 0; i < this._numberFormatTags.length; i++) {
                    if (_xlsx_number_format_helper2.default.areEqual(this._numberFormatTags[i], tag)) {
                        result = this._numberFormatTags[i][_xlsx_number_format_helper2.default.ID_PROPERTY_NAME];
                        break
                    }
                }
                if (void 0 === result) {
                    tag[_xlsx_number_format_helper2.default.ID_PROPERTY_NAME] = this._convertNumberFormatIndexToId(this._numberFormatTags.length);
                    result = tag[_xlsx_number_format_helper2.default.ID_PROPERTY_NAME];
                    this._numberFormatTags.push(tag)
                }
            }
            return result
        }
    }, {
        key: "generateNumberFormatsXml",
        value: function() {
            var _this = this;
            if (this._numberFormatTags.length > 0) {
                var xmlStringsArray = this._numberFormatTags.map(function(tag, index) {
                    return _xlsx_number_format_helper2.default.toXml(tag, function(attributes) {
                        return attributes.numFmtId = _this._convertNumberFormatIndexToId(index)
                    })
                });
                return _xlsx_tag_helper2.default.toXml("numFmts", {
                    count: xmlStringsArray.length
                }, xmlStringsArray.join(""))
            } else {
                return ""
            }
        }
    }]);
    return XlsxFile
}();
exports.default = XlsxFile;
module.exports = XlsxFile;
